#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <dht.h>

dht DHT;
LiquidCrystal_I2C lcd(0x27,16,2);
#define ADC_CHANNEL 0 // A0

int dhtPin = 7;
int buzzerPin = 11;

int redPin = 6;
int greenPin = 5;
int bluePin = 4;

void setup() {
  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);

  pinMode(buzzerPin, OUTPUT);

  lcd.init();                      
  lcd.backlight();

  // Configure the ADC
  // Clear registers
  ADCSRA = 0;
  ADCSRB = 0;
  ADMUX  = 0;

  // AVCC with external capacitor at AREF pin
  ADMUX |= (1 << REFS0);
  ADMUX |= ADC_CHANNEL;

  // Set ADC prescaler to 128 (16MHz/128 = 125kHz)
  ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);

  // Enable the ADC
  ADCSRA |= (1 << ADEN);
}

void loop() {
  // Start the ADC conversion
  ADCSRA |= (1 << ADSC);

  // Wait for the conversion to complete
  while (ADCSRA & (1 << ADSC));

  // Read the ADC value
  uint16_t adcValue = ADC;

  if (adcValue >= 700) {
    analogWrite(buzzerPin, 200);
    printWarningMessage();

  } else {
    if (adcValue >= 400) {
      setColor(255, 0, 0); // Red Color
      printGettingBad(adcValue);
      buzzerDelay(buzzerPin);

    } else {
      setColor(0, 255, 0); // Green Color

      int chk = DHT.read11(dhtPin);

      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Good air quality!");

      lcd.setCursor(0, 1);
      lcd.print("Temperature = ");
      lcd.print(DHT.temperature);
      delay(1000);
    }
  }
}

void buzzerDelay(int buzzerPin) {
  analogWrite(buzzerPin, 127);
  delay(500);
  analogWrite(buzzerPin, 0);
  delay(500);
}

void setColor(int redValue, int greenValue, int blueValue) {
  analogWrite(redPin, redValue);
  analogWrite(greenPin, greenValue);
  analogWrite(bluePin, blueValue);
}

void printWarningMessage() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("    WARNING   ");

  lcd.setCursor(0, 1);
  lcd.print("  DANGEROUS GAS  ");
  delay(1000);

  lcd.noDisplay();
  delay(500);
  lcd.display();
  delay(1000);
}

void printGettingBad(int adcValue) {
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.print("Air quality is ");

  lcd.setCursor(0,1);
  lcd.print("getting bad: ");
  lcd.print(adcValue);
  delay(500);
}